package com.example.user.aop;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.Date;

/**
 * 日志AOP，对具有@Log自定义注解的方法进行拦截处理
 */
@Component
@Slf4j
@Aspect
public class LogAOP {
    /**
     * 切入点，对注解@Log进行拦截
     */
    @Pointcut("@annotation(com.example.user.annotation.Log)")
    public void pointcut(){

    }

    /**
     * 环绕proceedingJoinPoint
     * @return
     */
    @Around("pointcut()")
    public Object around(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        Object result = null;

        //获取拦截的方法
        MethodSignature signature = (MethodSignature) proceedingJoinPoint.getSignature();
        Method method = signature.getMethod();
        //记录拦截方法调用前日志
        log.info(String.format("【AOP-LogAOP】Around：拦截方法:%s，开始时间:%s。",method.getName(),new Date().toString()));

        //拦截方法调用，此处可做一定的逻辑，如果是对权限进行过滤，没有权限不可调用，可return null
        result =  proceedingJoinPoint.proceed();

        //记录拦截方法调用后日志
        log.info(String.format("【AOP-LogAOP】Around：拦截方法:%s，结束时间:%s，返回结果：%s。",method.getName(),new Date().toString(),new ObjectMapper().writeValueAsString(result)));

        return result;
    }
}
