package com.example.user.controller;

import com.example.user.entity.UserEntity;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Project: user
 * @Author: sanbao
 * @Create: 2021-11-23 14:14
 * @Desc： 用户控制器，提供用户信息的CRUD。
 **/
@Tag(name = "用户信息")
@RestController
@RequestMapping("/user")
public class UserController {
    @Autowired
    List<UserEntity> userEntities;

    /**
     * 用户查询
     * @return
     */
    @Operation(tags = "查询",summary = "查询所有数据")
    @GetMapping("/")
    public List<UserEntity> query(){
        return this.userEntities;
    }

    /**
     * 根据ID获取数据
     * @param id
     * @return
     */
    //【AOP】5、应用到目标，通过@Pointcut配置自动生效
    @Operation(tags = "根据ID获取",summary = "根据ID获取详细信息")
    @GetMapping("/{id}")
    public UserEntity getById(@PathVariable Integer id){
        return this.userEntities.stream().filter(s->s.getId() == id).findFirst().orElse(null);
    }

    /**
     * 新增
     * @param userEntity
     */
    @Operation(tags = "新增",summary = "新增一条数据")
    @PostMapping("/")
    public void create(@RequestBody UserEntity userEntity){
        this.userEntities.add(userEntity);
    }

    /**
     * 修改
     * @param userEntity
     */
    @Operation(tags = "修改",summary = "修改一条数据")
    @PutMapping("/")
    public void update(@RequestBody UserEntity userEntity){
        //根据获取已经存在的数据
        UserEntity existUserEntity = this.userEntities.stream().filter(item->item.getId() == userEntity.getId()).findFirst().orElse(null);
        //拷贝属性，即更新
        BeanUtils.copyProperties(userEntity,existUserEntity);
    }

    /**
     * 删除
     * @param id
     */
    @Operation(tags = "删除",summary = "删除一条数据")
    @DeleteMapping("/{id}")
    public void deleteById(@PathVariable Integer id){
        //过滤出非当前id的所有数据，即实现删除
        this.userEntities = this.userEntities.stream().filter(s->s.getId() != id).collect(Collectors.toList());
    }
}